@@ -22,6 +22,10 @@
 #include "video.h"
 #include "video_blit.h"
 
+#if USE_SDL_VIDEO
+#include <SDL.h>
+#endif
+
 #include <stdio.h>
 #include <stdlib.h>
 
@@ -301,11 +305,11 @@ static void Blit_Copy_Raw(uint8 * dest, const uint8 * source, uint32 length)
 #define FB_DEPTH 24
 #include "video_blit.h"
 
+#if !(REAL_ADDRESSING || DIRECT_ADDRESSING || USE_SDL_VIDEO)
 /* -------------------------------------------------------------------------- */
 /* --- 1-bit indexed to 8-bit color mode conversion                       --- */
 /* -------------------------------------------------------------------------- */
 
-#if !REAL_ADDRESSING && !DIRECT_ADDRESSING
 #define CONVERT_BW(byte) (byte)==1?0:255
 static void Blit_Expand_1_To_8_Color(uint8 * dest, const uint8 * p, uint32 length)
 {
@@ -521,11 +525,19 @@ bool Screen_blitter_init(VisualFormat const & visual_format, bool native_byte_or
 #else
 	const bool use_sdl_video = false;
 #endif
-#if REAL_ADDRESSING || DIRECT_ADDRESSING
+#if REAL_ADDRESSING || DIRECT_ADDRESSING || USE_SDL_VIDEO
 	if (mac_depth == 1 && !use_sdl_video && !visual_format.fullscreen) {
 
 		// Windowed 1-bit mode uses a 1-bit X image, so there's no need for special blitting routines
 		Screen_blit = Blit_Copy_Raw;
+
+#if !DIRECT_ADDRESSING && defined(__aarch64__)
+	} else if (mac_depth == 16) {
+
+		Screen_blit = Blit_Copy_Raw;
+
+#endif
+	
 	} else {
 
 		// Compute RGB shift values
