@@ -43,6 +43,10 @@
 # include <sys/mman.h>
 #endif
 
+#if __MACOSX__
+# include "utils_macosx.h"
+#endif
+
 #if !EMULATED_68K && defined(__NetBSD__)
 # include <m68k/sync_icache.h> 
 # include <m68k/frame.h>
@@ -60,9 +64,6 @@ struct sigstate {
 #ifdef ENABLE_GTK
 # include <gtk/gtk.h>
 # include <gdk/gdk.h>
-# ifdef HAVE_GNOMEUI
-#  include <gnome.h>
-# endif
 # if !defined(GDK_WINDOWING_QUARTZ) && !defined(GDK_WINDOWING_WAYLAND)
 #  include <X11/Xlib.h>
 # endif
@@ -94,8 +95,13 @@ using std::string;
 #include "rpc.h"
 
 #if USE_JIT
+#ifdef UPDATE_UAE
+extern void (*flush_icache)(void); // from compemu_support.cpp
+extern bool UseJIT;
+#else
 extern void flush_icache_range(uint8 *start, uint32 size); // from compemu_support.cpp
 #endif
+#endif
 
 #ifdef ENABLE_MON
 # include "mon.h"
@@ -242,11 +248,12 @@ void *vm_acquire_mac(size_t size)
 	return vm_acquire(size, VM_MAP_DEFAULT | VM_MAP_32BIT);
 }
 
+#if REAL_ADDRESSING
 static int vm_acquire_mac_fixed(void *addr, size_t size)
 {
 	return vm_acquire_fixed(addr, size, VM_MAP_DEFAULT | VM_MAP_32BIT);
 }
-
+#endif
 
 /*
  *  SIGSEGV handler
@@ -289,9 +296,14 @@ static void sigsegv_dump_state(sigsegv_info_t *sip)
 	fprintf(stderr, "\n");
 #if EMULATED_68K
 	uaecptr nextpc;
+#ifdef UPDATE_UAE
+	extern void m68k_dumpstate(FILE *, uaecptr *nextpc);
+	m68k_dumpstate(stderr, &nextpc);
+#else
 	extern void m68k_dumpstate(uaecptr *nextpc);
 	m68k_dumpstate(&nextpc);
 #endif
+#endif
 #if USE_JIT && JIT_DEBUG
 	extern void compiler_dumpstate(void);
 	compiler_dumpstate();
@@ -364,6 +376,16 @@ void cpu_do_check_ticks(void)
 	if (emulated_ticks <= 0)
 		emulated_ticks += emulated_ticks_quantum;
 }
+#else
+uint16 emulated_ticks;
+void cpu_do_check_ticks(void)
+{
+	static int delay = -1;
+	if (delay < 0)
+		delay = PrefsFindInt32("delay");
+	if (delay)
+		usleep(delay);
+}
 #endif
 
 
@@ -385,6 +407,7 @@ static void usage(const char *prg_name)
 	);
 	LoadPrefs(NULL); // read the prefs file so PrefsPrintUsage() will print the correct default values
 	PrefsPrintUsage();
+	printf("\nBuild Date: %s\n", __DATE__);
 	exit(0);
 }
 
@@ -456,7 +479,7 @@ int main(int argc, char **argv)
 			argv[i] = NULL;
 		}
 		
-#if __MACOSX__
+#if defined(__APPLE__) && defined(__MACH__)
 		// Mac OS X likes to pass in various options of its own, when launching an app.
 		// Attempt to ignore these.
 		if (argv[i]) {
@@ -494,16 +517,9 @@ int main(int argc, char **argv)
 
 #ifdef ENABLE_GTK
 	if (!gui_connection) {
-#ifdef HAVE_GNOMEUI
-		// Init GNOME/GTK
-		char version[16];
-		sprintf(version, "%d.%d", VERSION_MAJOR, VERSION_MINOR);
-		gnome_init("Basilisk II", version, argc, argv);
-#else
 		// Init GTK
 		gtk_set_locale();
 		gtk_init(&argc, &argv);
-#endif
 	}
 #endif
 
@@ -559,7 +575,6 @@ int main(int argc, char **argv)
 	//
 	// HACK: disable these shortcuts, while leaving all other pieces of SDL2's
 	// menu bar in-place.
-	extern void disable_SDL2_macosx_menu_bar_keyboard_shortcuts();
 	disable_SDL2_macosx_menu_bar_keyboard_shortcuts();
 #endif
 	
@@ -584,7 +599,11 @@ int main(int argc, char **argv)
 	sigsegv_set_dump_state(sigsegv_dump_state);
 
 	// Read RAM size
-	RAMSize = PrefsFindInt32("ramsize") & 0xfff00000;	// Round down to 1MB boundary
+	RAMSize = PrefsFindInt32("ramsize");
+	if (RAMSize <= 1000) {
+		RAMSize *= 1024 * 1024;
+	}
+	RAMSize &= 0xfff00000;	// Round down to 1MB boundary
 	if (RAMSize < 1024*1024) {
 		WarningAlert(GetString(STR_SMALL_RAM_WARN));
 		RAMSize = 1024*1024;
@@ -986,8 +1005,12 @@ void FlushCodeCache(void *start, uint32 size)
 {
 #if USE_JIT
     if (UseJIT)
+#ifdef UPDATE_UAE
+		flush_icache();
+#else
 		flush_icache_range((uint8 *)start, size);
 #endif
+#endif
 #if !EMULATED_68K && defined(__NetBSD__)
 	m68k_sync_icache(start, size);
 #endif
@@ -1003,8 +1026,13 @@ static void sigint_handler(...)
 {
 #if EMULATED_68K
 	uaecptr nextpc;
+#ifdef UPDATE_UAE
+	extern void m68k_dumpstate(FILE *, uaecptr *nextpc);
+	m68k_dumpstate(stderr, &nextpc);
+#else
 	extern void m68k_dumpstate(uaecptr *nextpc);
 	m68k_dumpstate(&nextpc);
+#endif
 #endif
 	VideoQuitFullScreen();
 	const char *arg[4] = {"mon", "-m", "-r", NULL};
@@ -1241,7 +1269,7 @@ static void *tick_func(void *arg)
 {
 	uint64 start = GetTicks_usec();
 	int64 ticks = 0;
-	uint64 next = GetTicks_usec();
+	uint64 next = start;
 	while (!tick_thread_cancel) {
 		one_tick();
 		next += 16625;
@@ -1252,8 +1280,10 @@ static void *tick_func(void *arg)
 			next = GetTicks_usec();
 		ticks++;
 	}
+#if DEBUG
 	uint64 end = GetTicks_usec();
 	D(bug("%lld ticks in %lld usec = %f ticks/sec\n", ticks, end - start, ticks * 1000000.0 / (end - start)));
+#endif
 	return NULL;
 }
 #endif
