@@ -44,7 +44,7 @@
 #include "vm_alloc.h"
 #include "sigsegv.h"
 #include "util_windows.h"
-#include "kernel_windows.h"
+//#include "kernel_windows.h"
 
 #define DEBUG 0
 #include "debug.h"
@@ -105,6 +105,8 @@ uintptr SheepMem::base = 0x60000000;		// Address of SheepShaver data
 uintptr SheepMem::proc;						// Bottom address of SheepShave procedures
 uintptr SheepMem::data;						// Top of SheepShaver data (stack like storage)
 
+static HHOOK keyboard_hook;					// Hook for intercepting windows key events
+
 
 // Prototypes
 static bool kernel_data_init(void);
@@ -118,6 +120,7 @@ extern void emul_ppc(uint32 start);
 extern void init_emul_ppc(void);
 extern void exit_emul_ppc(void);
 sigsegv_return_t sigsegv_handler(sigsegv_info_t *sip);
+static LRESULT CALLBACK low_level_keyboard_hook(int nCode, WPARAM wParam, LPARAM lParam);
 
 
 /*
@@ -155,6 +158,7 @@ static void usage(const char *prg_name)
 	printf("\nUnix options:\n");
 	printf("  --display STRING\n    X display to use\n");
 	PrefsPrintUsage();
+	printf("\nBuild Date: %s\n", __DATE__);
 	exit(0);
 }
 
@@ -175,19 +179,26 @@ int main(int argc, char **argv)
 	printf(GetString(STR_ABOUT_TEXT1), VERSION_MAJOR, VERSION_MINOR);
 	printf(" %s\n", GetString(STR_ABOUT_TEXT2));
 
-	// Read preferences
-	PrefsInit(NULL, argc, argv);
-
 	// Parse command line arguments
 	for (int i=1; i<argc; i++) {
 		if (strcmp(argv[i], "--help") == 0) {
 			usage(argv[0]);
+		} else if (strcmp(argv[i], "--config") == 0) {
+			argv[i++] = NULL;
+			if (i < argc) {
+				extern std::string UserPrefsPath; // from prefs_windows.cpp
+				UserPrefsPath = to_tstring(argv[i]);
+				argv[i] = NULL;
+			}
 		} else if (argv[i][0] == '-') {
 			fprintf(stderr, "Unrecognized option '%s'\n", argv[i]);
 			usage(argv[0]);
 		}
 	}
 
+	// Read preferences
+	PrefsInit(NULL, argc, argv);
+
 	// Check we are using a Windows NT kernel >= 4.0
 	OSVERSIONINFO osvi;
 	ZeroMemory(&osvi, sizeof(OSVERSIONINFO));
@@ -207,12 +218,14 @@ int main(int argc, char **argv)
 	if (!check_drivers())
 		QuitEmulator();
 
-	// Load win32 libraries
-	KernelInit();
+//	// Load win32 libraries
+//	KernelInit();
 
-	// FIXME: default to DIB driver
-	if (getenv("SDL_VIDEODRIVER") == NULL)
-	    putenv("SDL_VIDEODRIVER=windib");
+	// Install keyboard hook to block Windows key if enabled in prefs
+	if (PrefsFindBool("reservewindowskey"))
+	{
+		keyboard_hook = SetWindowsHookEx(WH_KEYBOARD_LL, low_level_keyboard_hook, GetModuleHandle(NULL), 0);
+	}
 
 	// Initialize SDL system
 	int sdl_flags = 0;
@@ -253,6 +266,10 @@ int main(int argc, char **argv)
 	TimebaseSpeed =  25000000;	// Default:  25MHz
 	PVR = 0x000c0000;			// Default: 7400 (with AltiVec)
 	D(bug("PVR: %08x (assumed)\n", PVR));
+	{
+		int pref_cpu_clock = PrefsFindInt32("cpuclock");
+		if (pref_cpu_clock) CPUClockSpeed = 1000000 * pref_cpu_clock;
+	}
 
 	// Init system routines
 	SysInit();
@@ -307,9 +324,12 @@ int main(int argc, char **argv)
 
 	// Create area for Mac RAM
 	RAMSize = PrefsFindInt32("ramsize");
-	if (RAMSize < 8*1024*1024) {
+	if (RAMSize <= 1000) {
+		RAMSize *= 1024 * 1024;
+	}
+	if (RAMSize < 16 * 1024 * 1024) {
 		WarningAlert(GetString(STR_SMALL_RAM_WARN));
-		RAMSize = 8*1024*1024;
+		RAMSize = 16 * 1024 * 1024;
 	}
 	RAMBase = 0;
 	if (vm_mac_acquire(RAMBase, RAMSize) < 0) {
@@ -446,8 +466,8 @@ static void Quit(void)
 	// Exit preferences
 	PrefsExit();
 
-	// Release win32 libraries
-	KernelExit();
+//	// Release win32 libraries
+//	KernelExit();
 
 #ifdef ENABLE_MON
 	// Exit mon
@@ -770,11 +790,21 @@ void SheepMem::Exit(void)
 
 #ifdef USE_SDL_VIDEO
 #include <SDL_syswm.h>
+extern SDL_Window *sdl_window;
 HWND GetMainWindowHandle(void)
 {
 	SDL_SysWMinfo wmInfo;
 	SDL_VERSION(&wmInfo.version);
-	return SDL_GetWMInfo(&wmInfo) ? wmInfo.window : NULL;
+	if (!sdl_window) {
+		return NULL;
+	}
+	if (!SDL_GetWindowWMInfo(sdl_window, &wmInfo)) {
+		return NULL;
+	}
+	if (wmInfo.subsystem != SDL_SYSWM_WINDOWS) {
+		return NULL;
+	}
+	return wmInfo.info.win.window;
 }
 #endif
 
@@ -826,3 +856,43 @@ bool ChoiceAlert(const char *text, const char *pos, const char *neg)
 	printf(GetString(STR_SHELL_WARNING_PREFIX), text);
 	return false;	//!!
 }
+
+/*
+ *  Low level keyboard hook allowing us to intercept events involving the Windows key
+ */
+static LRESULT CALLBACK low_level_keyboard_hook(int nCode, WPARAM wParam, LPARAM lParam)
+{
+	// Not a relevant event, immediately pass it on
+	if (nCode != HC_ACTION)
+        return CallNextHookEx(keyboard_hook, nCode, wParam, lParam);
+
+	KBDLLHOOKSTRUCT *p = (KBDLLHOOKSTRUCT *)lParam;
+	switch (wParam) {
+		case WM_KEYDOWN:
+		case WM_KEYUP:
+			// Intercept left/right windows keys when we have keyboard focus so Windows doesn't handle them
+			if (p->vkCode == VK_LWIN || p->vkCode == VK_RWIN) {
+				bool intercept_event = false;
+#ifdef USE_SDL_VIDEO
+				if (sdl_window && (SDL_GetWindowFlags(sdl_window) & SDL_WINDOW_INPUT_FOCUS)) {
+					intercept_event = true;
+				}
+#endif
+
+				// If we've determined we should intercept the event, intercept it. But pass the event onto SDL so SheepShaver handles it.
+				if (intercept_event) {
+					SDL_Event e;
+					memset(&e, 0, sizeof(e));
+					e.type = (wParam == WM_KEYDOWN) ? SDL_KEYDOWN : SDL_KEYUP;
+					e.key.keysym.sym = (p->vkCode == VK_LWIN) ? SDLK_LGUI : SDLK_RGUI;
+					e.key.keysym.scancode = (p->vkCode == VK_LWIN) ? SDL_SCANCODE_LGUI : SDL_SCANCODE_RGUI;
+					SDL_PushEvent(&e);
+					return 1;
+				}
+			}
+			break;
+	}
+
+	// If we fall here, we weren't supposed to intercept it.
+	return CallNextHookEx(keyboard_hook, nCode, wParam, lParam);
+}
